//
//  SBGameScene.m
//  SBGame
//
//  Created by Xiaoxuan Tang on 12-11-23.
//
//

#import "SBGameScene.h"
#import "cocos2d.h"
#import "SimpleAudioEngine.h"
#import "SBGlobal.h"
#import "SBSprite.h"
#import "SBGlobal.h"
#import "SBCinematicSecond.h"
#import "SBLoseScene.h"
@implementation SBGameScene

- (id) initWithLevel:(NSInteger) level
{
    if ( self = [super init] )
    {
        [SBGlobal sharedGlobal].money = 100;
        
        if (![[SimpleAudioEngine sharedEngine] isBackgroundMusicPlaying])
        {
            [[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"play.mp3" loop:YES];
            [SBGlobal sharedGlobal].bgmName = @"play.mp3";
        }
        else
            if (![[SBGlobal sharedGlobal].bgmName isEqualToString:@"play.mp3"])
            {
                [[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"play.mp3" loop:YES];
                [SBGlobal sharedGlobal].bgmName = @"play.mp3";
            }
        
        
        CGSize winSize = [CCDirector sharedDirector].winSize;
        if (winSize.height > winSize.width)
        {
            CGFloat tmp = winSize.width;
            winSize.width = winSize.height;
            winSize.height = tmp;
        }
        
        gameLayer = [[SBGameLayer alloc] initWithLevel: level];
        gameLayer.position = ccp(0, 0);
        [self addChild:gameLayer];
        [gameLayer release];
        
        menuLayer = [[SBMenuLayer alloc] initWithLevel:level];
        menuLayer.position = ccp(0, 0);
        [self addChild:menuLayer];
        [menuLayer release];
        
        leftArmy = [[NSMutableArray alloc] init];
        rightArmy = [[NSMutableArray alloc] init];
        
        NSDictionary* castleDic = @{@"type":@"castle",
        @"png":@"left.png",
        @"attackRange": @"0",
        @"moveRange":@"0",
        @"damage":@"0",
        @"hp":@"100",
        @"left":@"YES",
        @"pos":NSStringFromCGPoint(ccp(350,200))};
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(addSprite:)
                                                     name:@"addSprite"
                                                   object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(endAttack:)
                                                     name:@"endAttack"
                                                   object:nil];
        [self addSpriteWithDict:castleDic];
        
        NSDictionary* boss = @{@"type": @"boss",
        @"attackRange": @"20",
        @"moveRange": @"0",
        @"damage":@"20",
        @"hp":@"200",
        @"left":@"NO",
        @"png":@"snz_stand_0.png",
        @"pos":NSStringFromCGPoint(ccp(2200, 250))};
        [self addSpriteWithDict:boss];
        
        
        [self schedule:@selector(ticker) interval:.1];
        [self genArmy];
        attack = [[NSMutableArray alloc] init];
        cnt = 0;
    }
    
    return self;
}

- (BOOL) attack:(BOOL) Left andSprte:(SBSprite*) sprite
{
    if (Left)
    {        
        for (SBSprite* tmp in rightArmy) {
            if (tmp.deadFlag) continue;
            if (sprite.position.x + sprite.attackRange >= tmp.position.x)
            {
                [attack addObject:@{@"source": sprite, @"target": tmp}];
                [sprite stopAllActions];
                [sprite runAction:[sprite attack]];
                return YES;
            }
        }
    }
    else
    {
        for (SBSprite* tmp in leftArmy) {
            if (tmp.deadFlag) continue;
            if (sprite.position.x - sprite.attackRange <= tmp.position.x)
            {
                [attack addObject:@{@"source": sprite, @"target": tmp}];
                [sprite stopAllActions];
                [sprite runAction:[sprite attack]];
                return YES;
            }
        }
    }
    return NO;
}

- (void) ticker
{
    for (SBSprite* sprite in leftArmy) {
        if (!sprite.attackFlag && !sprite.deadFlag)
        {
            if (![self attack:YES andSprte:sprite])
            {
                CGPoint point = sprite.position;
                point.x  += sprite.moveRange/2;
                sprite.position = point;
            }
        }
    }
    
    for (SBSprite* sprite in rightArmy) {
        if (!sprite.attackFlag && !sprite.deadFlag)
        {
            if (![self attack:NO andSprte:sprite])
            {
                CGPoint point = sprite.position;
                point.x  -= sprite.moveRange/2;
                sprite.position = point;
            }
        }
    }
    
    cnt ++;
    if (cnt > 100)
    {
        cnt -= 100;
        [self genArmy];
    }
}

- (void) endAttack: (NSNotification*) notification
{
    SBSprite* sprite = notification.userInfo[@"source"];
    
    NSDictionary* dict;
    for (NSDictionary* _dict in attack) {
        if (_dict[@"source"] == sprite)
        {
            dict = _dict;
        }
    }
    
    if (!sprite.deadFlag)
    {
        SBSprite* tmp = dict[@"target"];
        if (!tmp.deadFlag)
        {
            tmp.hp -= sprite.damage;
            if (tmp.hp <= 0)
            {
                if ([tmp.type isEqualToString:@"boss"])
                {
                    [self win];
                }
                
                if ([tmp.type isEqualToString:@"fire_actor"])
                {
                    [SBGlobal sharedGlobal].money += 25;
                    [menuLayer moneyChange];
                }
                
                if ([tmp.type isEqualToString:@"water_actor"])
                {
                    [SBGlobal sharedGlobal].money += 25;
                    [menuLayer moneyChange];
                }
                
                if ([tmp.type isEqualToString:@"sand_actor"])
                {
                    [SBGlobal sharedGlobal].money += 25;
                    [menuLayer moneyChange];
                }
                
                if ([tmp.type isEqualToString:@"castle"])
                {
                    [self lose];
                }
                [tmp dead];
            }
        }
        [sprite runAction:[sprite walk]];
    }
    
    [attack removeObject:dict];
}

- (void) lose
{
    [[CCDirector sharedDirector] replaceScene:[CCTransitionFade transitionWithDuration:.5f
                                                                                 scene:[SBLoseScene node]]];
}

- (void) win
{
    UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"Congratulation"
                                                    message:@"You Win!"
                                                   delegate:nil
                                          cancelButtonTitle:@"I know it is easy"
                                          otherButtonTitles: nil];
    [alert show];
    [alert release];
    
    [[CCDirector sharedDirector] replaceScene:[CCTransitionFade transitionWithDuration:.5f
                                                                                 scene:[SBCinematicSecond node]]];
}

- (void) genArmy
{
    int tmp = arc4random();
    if (tmp % 3 == 0)
        [self addSpriteWithType:@"fire_actor"];
    if (tmp % 3 == 1)
        [self addSpriteWithType:@"sand_actor"];
    if (tmp % 3 == 2)
        [self addSpriteWithType:@"water_actor"];
}

- (void) addSpriteWithType: (NSString*) type
{
    SBSprite* sprite = [[SBSprite alloc] initWithType:type];
    sprite.position = ccp(2200, 250);
    [gameLayer addChild:sprite];
    [sprite runAction:[sprite walk]];
    [rightArmy addObject:sprite];
    [sprite release];
}

- (void) addSprite:(NSNotification*) notification
{
    SBSprite* sprite = [[SBSprite alloc] initWithType:notification.userInfo[@"type"]];
    sprite.position = ccp(300, 250);
    [gameLayer addChild:sprite];
    [sprite runAction:[sprite walk]];
    [leftArmy addObject:sprite];
    [sprite release];
}

- (void) addSpriteWithDict: (NSDictionary*) dict
{
    SBSprite* sprite = [[SBSprite alloc] initWithDicitionary:dict];
    if ([dict[@"left"] boolValue])
    {
        [leftArmy addObject:sprite];
        sprite.anchorPoint = ccp(1, 0);
        if (dict[@"pos"] == nil)
        {
            
        }
        else
            sprite.position = CGPointFromString(dict[@"pos"]);
    }
    else
    {
        sprite.anchorPoint = ccp(0, 0);
        if (dict[@"pos"] == nil)
        {
            
        }
        else
            sprite.position = CGPointFromString(dict[@"pos"]);
        [rightArmy addObject:sprite];
        [sprite runAction:[sprite walk]];
    }
    
    [gameLayer addChild:sprite];
    [sprite release];
}


@end
