//
//  SBSprite.m
//  SBGame
//
//  Created by Xiaoxuan Tang on 12-11-23.
//
//

#import "SBSprite.h"

@implementation SBSprite

- (id) initWithDicitionary:(NSDictionary *)dict
{
    if  ( self = [super initWithSpriteFrameName:dict[@"png"]] )
    {
        _attackRange = [dict[@"attackRange"] intValue];
        _moveRange   = [dict[@"moveRange"] intValue];
        _damage      = [dict[@"damage"] intValue];
        _hp          = [dict[@"hp"] intValue];
        _type        = [dict[@"type"] retain];
        _hpMax       = _hp;
        
        _hpStick = [CCLayerColor layerWithColor:ccc4(0, 255, 0, 255)
                                          width:self.boundingBox.size.width
                                         height:10];
        _hpStick.position = ccp(0, self.boundingBox.size.height);
        [self addChild:_hpStick];
    }
    return self;
}

- (id) initWithType:(NSString *)type
{
    NSDictionary* dict = nil;
    if ([type isEqualToString:@"major"])
    {
        dict = @{@"type" : type,
                @"attackRange": @"20",
                @"moveRange": @"20",
                @"damage":@"30",
                @"hp":@"300",
                @"left":@"YES",
                @"png":@"major_stand.png"};
        self = [self initWithDicitionary:dict];
    }
    
    if ([type isEqualToString:@"cy"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"20",
        @"moveRange": @"20",
        @"damage":@"30",
        @"hp":@"100",
        @"left":@"YES",
        @"png":@"cy_stand.png"};
        self = [self initWithDicitionary:dict];
    }
    
    if ([type isEqualToString:@"fire"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"15",
        @"moveRange": @"15",
        @"damage":@"12",
        @"hp":@"20",
        @"left":@"YES",
        @"png":@"fire_stand.png"};
        self = [self initWithDicitionary:dict];
    }
    if ([type isEqualToString:@"water"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"10",
        @"moveRange": @"10",
        @"damage":@"10",
        @"hp":@"25",
        @"left":@"YES",
        @"png":@"water_stand.png"};
        self = [self initWithDicitionary:dict];
    }
    if ([type isEqualToString:@"sand"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"15",
        @"moveRange": @"15",
        @"damage":@"15",
        @"hp":@"12",
        @"left":@"YES",
        @"png":@"sand_walk_1.png"};
        self = [self initWithDicitionary:dict];
    }
    
    if ([type isEqualToString:@"fire_actor"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"15",
        @"moveRange": @"15",
        @"damage":@"5",
        @"hp":@"20",
        @"left":@"NO",
        @"png":@"fire_actor_stand.png"};
        self = [self initWithDicitionary:dict];
    }
    if ([type isEqualToString:@"water_actor"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"10",
        @"moveRange": @"10",
        @"damage":@"3",
        @"hp":@"25",
        @"left":@"NO",
        @"png":@"water_actor_stand.png"};
        self = [self initWithDicitionary:dict];
    }
    if ([type isEqualToString:@"sand_actor"])
    {
        dict = @{@"type" : type,
        @"attackRange": @"15",
        @"moveRange": @"15",
        @"damage":@"5",
        @"hp":@"10",
        @"left":@"NO",
        @"png":@"sand_actor_walk_1.png"};
        self = [self initWithDicitionary:dict];
    }
    
    if ([type isEqualToString:@"boss"])
    {
        dict = @{@"type": type,
        @"attackRange": @"15",
        @"moveRange": @"0",
        @"damage":@"10",
        @"hp":@"200",
        @"left":@"NO",
        @"png":@"snz_stand_0.png"};
        self = [self initWithDicitionary:dict];
    }
    return self;
}

- (void) setHp:(NSInteger)hp
{
    _hp = hp;
    if (hp <=0 ) hp = 0;
    
    _hpStick.scaleX = (CGFloat)_hp/_hpMax;
    if (_hpStick.scaleX < .5) _hpStick.color = ccc3(255, 148, 0);
    if (_hpStick.scaleX < .2) _hpStick.color = ccc3(255, 0, 0);
}

- (void) dead
{
    _deadFlag = YES;
    [self stopAllActions];
    [self runAction:[CCSequence actionOne:[CCFadeOut actionWithDuration:.8]
                                      two:[CCCallFuncN actionWithTarget:self
                                                               selector:@selector(remove)]]];
}

- (CCAction* ) walk
{
    CCAction* action = nil;
    if ([_type isEqualToString:@"major"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<4; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"major_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"cy"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"cy_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"water"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                spriteFrameByName:[NSString stringWithFormat:@"water_stand.png"]];
        [animation addSpriteFrame: frame];
        
        frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                spriteFrameByName:[NSString stringWithFormat:@"water_walk.png"]];
        [animation addSpriteFrame: frame];

        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"sand"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<4; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"sand_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"fire"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<4; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"fire_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"fire_actor"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"fire_actor_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"sand_actor"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"sand_actor_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"water_actor"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"water_actor_walk_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    
    if ([_type isEqualToString:@"boss"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=0; i<5; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"snz_stand_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCRepeatForever* repeat = [CCRepeatForever actionWithAction:animate];
        action = repeat;
    }
    return action;
}

- (void) startAttack
{
    _attackFlag = YES;
    CGPoint pos = self.position;
    pos.x    += _moveRange;
    self.position  = pos;
}

- (void) endAttack
{
    _attackFlag = NO;
    CGPoint pos = self.position;
    pos.x   -= _moveRange;
    self.position = pos;
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"endAttack"
                                                        object:nil
                                                      userInfo:@{@"source": self}];
}

- (CCAction*) attack
{
    CCAction* action;
    if ([_type isEqualToString:@"major"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"major_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                                                animate,
                                                [CCCallFunc actionWithTarget:self
                                                                    selector:@selector(endAttack)], nil];
        action = seq;
    }

    if ([_type isEqualToString:@"cy"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"cy_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    
    if ([_type isEqualToString:@"fire"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<4; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"fire_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    
    if ([_type isEqualToString:@"sand"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<5; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"sand_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    
    if ([_type isEqualToString:@"water"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<5; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"water_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    
    if ([_type isEqualToString:@"boss"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<7; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"snz_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    if ([_type isEqualToString:@"fire_actor"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"fire_actor_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    if ([_type isEqualToString:@"water_actor"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;
        for (int i=1; i<3; i++) {
            CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                    spriteFrameByName:[NSString stringWithFormat:@"water_actor_attack_%d.png", i]];
            [animation addSpriteFrame: frame];
        }
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    
    if ([_type isEqualToString:@"sand_actor"])
    {
        CCAnimation* animation = [CCAnimation animation];
        animation.delayPerUnit = .25;

        CCSpriteFrame* frame = [[CCSpriteFrameCache sharedSpriteFrameCache]
                                spriteFrameByName:[NSString stringWithFormat:@"sand_actor_attack.png"]];
        [animation addSpriteFrame: frame];
        
        CCAnimate *animate = [CCAnimate actionWithAnimation:animation];
        CCSequence * seq = [CCSequence actions:[CCCallFunc actionWithTarget:self
                                                                   selector:@selector(startAttack)],
                            animate,
                            [CCCallFunc actionWithTarget:self
                                                selector:@selector(endAttack)], nil];
        action = seq;
    }
    return action;
}

- (void) remove
{
    _deadFlag = YES;
    [self removeFromParentAndCleanup:YES];
}
@end
